// lowglc_3.cmd
// By Yongkang Chen, Portland State University
//
// command scripts for low gravity liquid configuration
//
// History of modification:
// Sept. 11, 2006 original version
//
quiet on
//

x_max := 1e6
y_max := 1e6
z_max := 1e6
x_min := -1e6
y_min := -1e6
z_min := -1e6
//
vertex_number := 0
lam_new:=0             // for the operation of changing aspect ratio of the tank
lam_old:=0             // for the operation of changing aspect ratio of the tank
bond_value_idx := 1    // indicate whether bond value take regular increment or specified values. 1: specified values
hessian_idx := 0       // indicate whether perform Hessian stuff, 0 no, 1 Yes
hessian_M_idx  := 1       // indicate whether upgrade the mesh to quadratic mode. 1: M1; 2: M2

//
error_idx := 1;
set_error := {if error_idx == 1 then {error_ori := pow(10, (-6 + floor(log(abs(total_energy))/log(10))))}; error := error_ori} 
set_error
//
lr      := max(edge where not fixed, length)
lr_max  := 0
my_re   := {refine edge where not fixed and length >lr}

vert_fix_idx := 0
vert_fix := {}
vert_unfix := {}

// script to set up filename
// name index name_idx, 2 for Bond increment as phi is fixed. 
// Otherwise it is for phi increment as Bond is fixed.
name_idx := 2
phi_datafile_head_idx := 1
bond_datefile_head_idx := 1
do_name := {eta_s := sprintf "%g", eta; bond_s := sprintf "%g", bond; phi_s := sprintf "%g", phi;
            name_root1 := sprintf "LowGLC_eta_%s_Bo_%s_phi_%s",eta_s,bond_s,phi_s; 
	    if name_idx == 2 then { name_root2 := sprintf "LowGLC_eta_%s_Bo_%s",eta_s,bond_s} 
	    else {name_root2 := sprintf "LowGLC_eta_%s_phi_%s",eta_s,phi_s};
	    dumpname := sprintf "%s.dmp",name_root1; print dumpname;
            logname  := sprintf "%s.log",name_root1;
            facname  := sprintf "%s.fac",name_root1;
            edgname  := sprintf "%s.edg",name_root1;
            vername  := sprintf "%s.ver",name_root1;
            geomname := sprintf "%s.dat",name_root2; 
            eigname := sprintf "%s.eig",name_root2;
            if name_idx == 2 and phi_datafile_head_idx == 1 then {
            printf "   Bond     phi        lambda     eta     dx          dy          dz          xmc         ymc         zmc       dry_area\n" >> geomname;
            phi_data_head_idx :=0};
            if name_idx == 3 and bond_datefile_head_idx == 1 then {
            printf "   Bond     phi        lambda     eta     dx          dy          dz          xmc         ymc         zmc       dry_area\n" >> geomname;
            bond_datefile_head_idx := 0}
	    }
do_name

do_dump := {do_name; dump dumpname}
//do_dump := {}
//
// script to print the dimensions and cent of mass information
//
do_dim  := {delta_x := max(vertex where not fixed, x)- min(vertex where not fixed, x);
	        delta_y := max(vertex where not fixed, y)- min(vertex where not fixed, y);
	        delta_z := max(vertex where not fixed, z)- min(vertex where not fixed, z)
	       }
dry_spot := {dry_area := 0; set facet where not fixed color red; set edge where not fixed color black;
             foreach facet ff where not fixed do
             {if abs(ff.vertex[1].z) > lambda/2 then sum1 := ff.vertex[1].x^2+ff.vertex[1].y^2+(abs(ff.vertex[1].z)-lambda/2)^2
               else sum1 := ff.vertex[1].x^2+ff.vertex[1].y^2;
               if abs(ff.vertex[2].z) > lambda/2 then sum2 := ff.vertex[2].x^2+ff.vertex[2].y^2+(abs(ff.vertex[2].z)-lambda/2)^2
               else sum2 := ff.vertex[2].x^2+ff.vertex[2].y^2;
               if abs(ff.vertex[3].z) > lambda/2 then sum3 := ff.vertex[3].x^2+ff.vertex[3].y^2+(abs(ff.vertex[3].z)-lambda/2)^2
               else sum3 := ff.vertex[3].x^2+ff.vertex[3].y^2;
              if sum1>0.999 and sum2>0.999 and sum3>0.999 then {
                 set ff color clear;  
                 print ff.id; dry_area := dry_area + ff.area};
	         };
                 foreach edge ee where not fixed do { if ee.facet[1].color == clear and ee.facet[2].color == clear then set ee color clear};
                 foreach edge ee where not fixed and color != clear do {if ee.facet[1].color == clear or ee.facet[2].color == clear then set ee color green};
                 set edge where not fixed and color != green color clear;
	             printf "Total interface area : %12.8f\n", sum(facet where not fixed, area);
	             printf "           Dry spot area : %12.8f\n", dry_area;
	         }
//
dim_print := {do_dim;
              printf "dx: %11.6f  dy: %11.6f  dz: %11.6f\n", delta_x, delta_y, delta_z;
              printf "  dx: %11.6f  dy: %11.6f  dz: %11.6f\n", delta_x, delta_y, delta_z >> logname;
	         }
geom_print := {do_dim; dry_spot;
	       printf "                         Bond : %11.7f\n", bond;
               printf "                            phi : %8.4f\n ", phi;
               printf "                    lambda : %8.4f\n ", lambda;
               printf "                            eta : %8.4f\n ", eta;
               printf "                             dx : %11.6f\n", delta_x;
               printf "                              dy : %11.6f\n", delta_y;
               printf "                              dz : %11.6f\n", delta_z;
               printf "                           xmc : %11.6f\n", xmc.value/body[1].target;
               printf "                           ymc : %11.6f\n", ymc.value/body[1].target;
               printf "                           zmc : %11.6f\n", zmc.value/body[1].target; 
               printf "%10.7f %8.4f %8.4f %8.4f %11.6f %11.6f %11.6f %11.6f %11.6f %11.6f %11.6f\n", bond, phi, lambda, eta, delta_x, delta_y, delta_z,  
	       xmc.value/body[1].target, ymc.value/body[1].target, zmc.value/body[1].target, dry_area >> geomname}
//
z_vert_adj_cri := 0    // the value of z for toggling on the vert_adj operation.
constr_adj_idx := 0
vert_adj := {}
vert_adj_ind := 0
topo_change := {}

//
fp:= foreach facet ff do {printf "%d\t %8d\t %8d\t %8d\n", ff.id, ff.edge[1].oid, ff.edge[2].oid, ff.edge[3].oid}
do_fp := {fp >> facname}
ep:= foreach edge ee do {printf"%d\t %8d\t %8d\n", ee.id, ee.vertex[1].id, ee.vertex[2].id}
do_ep := {ep >> edgname}
vp:= foreach vertex vv do {printf"%d\t %11.8f\t %11.8f\t %11.8f\n", vv.id, vv.x, vv.y, vv.z}
do_vp := {vp >> vername}
//
// Scripts for convergence
cons_adj := {for(inx:=14; inx<17; inx+=1)
             {unset vertex where on_constraint inx constraint inx; 
              unset edge  where  on_constraint inx constraint inx; 
              unset facet where  on_constraint inx constraint inx
             };
             foreach vertex vv where not fixed and vv.z > lambda/2 do {set vv constraint 16};
             foreach vertex vv where not fixed and abs(vv.z) < lambda/2 do {set vv constraint 15};
             foreach vertex vv where not fixed and vv.z < -lambda/2 do {set vv constraint 14};
             for(inx:=14; inx<17; inx+=1)
             {foreach edge ee where not fixed and ee.vertex[1] on_constraint inx do {set ee constraint inx};
              foreach edge ee where not fixed and not on_constraint inx and ee.vertex[2] on_constraint inx do {set ee constraint inx};
              foreach facet ff where not fixed and ff.edge[1] on_constraint inx do {set ff constraint inx};
              foreach facet ff where not fixed and not on_constraint inx and ff.edge[2] on_constraint inx do {set ff constraint inx};
              foreach facet ff where not fixed and not on_constraint inx and ff.edge[3] on_constraint inx do {set ff constraint inx};
             };
            } 
cons_rel := {for(inx:=14; inx<17; inx+=1)
             {unset vertex where on_constraint inx constraint inx; 
              unset edge  where  on_constraint inx constraint inx; 
              unset facet where  on_constraint inx constraint inx
             };
             set vertex where not fixed constraint 15;
	         set edge where not fixed constraint 15;
             set facet where not fixed constraint 15;
            } 
// 
cons_adj_idx:=1    // there is problem if cons_adj_idx = 0
my_re_idx := 0
my_re   := {refine edge where not fixed and length >lr}
number_iteration := 0
go_vug  := {D; V; u; g20; number_iteration += 1; if cons_adj_idx == 1 then cons_adj; if my_re_idx == 1 then my_re; D}
//
// change fill ratio
//
fill_ratio_op := {set body[1].target (1-eta)*(pi*(4/3+lambda))}
//
// extra_man locates the vertex on the container walls and change the color of 
// the facets on which the vertices are
//
extra_man := {set facet where not fixed color red; 
              set edge where not fixed color black;
              foreach facet ff where not fixed do
              {if abs(ff.vertex[1].z) > lambda/2 then sum1 := ff.vertex[1].x^2+ff.vertex[1].y^2+(abs(ff.vertex[1].z)-lambda/2)^2
               else sum1 := ff.vertex[1].x^2+ff.vertex[1].y^2;
               if abs(ff.vertex[2].z) > lambda/2 then sum2 := ff.vertex[2].x^2+ff.vertex[2].y^2+(abs(ff.vertex[2].z)-lambda/2)^2
               else sum2 := ff.vertex[2].x^2+ff.vertex[2].y^2;
               if abs(ff.vertex[3].z) > lambda/2 then sum3 := ff.vertex[3].x^2+ff.vertex[3].y^2+(abs(ff.vertex[3].z)-lambda/2)^2
               else sum3 := ff.vertex[3].x^2+ff.vertex[3].y^2;
               if sum1>0.999 and sum2>0.999 and sum3>0.999 then {
                set ff color clear}
               };
                 foreach edge ee where not fixed do { if ee.facet[1].color == clear and ee.facet[2].color == clear then set ee color clear};
                 foreach edge ee where not fixed and color != clear do {if ee.facet[1].color == clear or ee.facet[2].color == clear then set ee color green};
                 set edge where not fixed and color != green color clear;
              }
//              
//tank wall display
//
q_1_o := {set facet where original == 2 color -1; set facet where original == 6 color -1; set facet where original == 10 color -1 }
q_1_c := {set facet where original == 2 color lightgray; set facet where original == 6 color lightgray; set facet where original == 10 color lightgray }
q_2_o := {set facet where original == 3 color -1; set facet where original == 7 color -1; set facet where original == 11 color -1 }
q_2_c := {set facet where original == 3 color lightgray; set facet where original == 7 color lightgray; set facet where original == 11 color lightgray }
q_3_o := {set facet where original == 4 color -1; set facet where original == 8 color -1; set facet where original == 12 color -1 }
q_3_c := {set facet where original == 4 color lightgray; set facet where original == 8 color lightgray; set facet where original == 12 color lightgray }
q_4_o := {set facet where original == 1 color -1; set facet where original == 5 color -1; set facet where original == 9 color -1 }
q_4_c := {set facet where original == 1 color lightgray; set facet where original == 5 color lightgray; set facet where original == 9 color lightgray }
l_o := {set facet where original == 1 color -1; set facet where original == 2 color -1; set facet where original == 3 color -1;
          set facet where original == 4 color -1}
l_c := {set facet where original == 1 color lightgray; set facet where original == 2 color lightgray; set facet where original == 3 color lightgray;
          set facet where original == 4 color lightgray}
m_o := {set facet where original == 5 color -1; set facet where original == 6 color -1; set facet where original == 7 color -1;
          set facet where original == 8 color -1}
m_c := {set facet where original == 5 color lightgray; set facet where original == 6 color lightgray; set facet where original == 7 color lightgray;
          set facet where original == 8 color lightgray}
u_o := {set facet where original == 9 color -1; set facet where original == 10 color -1; set facet where original == 11 color -1;
          set facet where original == 12 color -1}
u_c := {set facet where original == 9 color lightgray; set facet where original == 10 color lightgray; set facet where original == 11 color lightgray;
          set facet where original == 12 color lightgray}
//
// initialization of the tank surface
//
ini_op := {{refine edge where not on_constraint 14; u}3; 
            set edge where not on_constraint 14 no_refine; set edge where not on_constraint 14 fixed;
            set facet where original < 13 fixed; set facet where fixed color -1; 
            set vertex where on_constraint 1 fixed; set vertex where on_constraint 2 fixed;
            set vertex where on_constraint 3 fixed; set vertex where on_constraint 4 fixed; 
            set vertex where on_constraint 5 fixed; set vertex where on_constraint 7 fixed;
            /*set vertex where on_constraint 12 fixed; set vertex where on_constraint 13 fixed*/}
//
//change the aspect ratio
//
ar_pre_op := {foreach vertex vv where on_constraint 5 do {unset vv constraint 5; set vv.z z+(lam_new-lam_old)/2}; 
              foreach vertex vv where on_constraint 1 do {unset vv constraint 1; set vv.z z-(lam_new-lam_old)/2};
              foreach vertex vv where on_constraint 2 do {unset vv constraint 2; set vv.z z-(lam_new-lam_old)/2};
              foreach vertex vv where on_constraint 4 do {unset vv constraint 4; set vv.z z+(lam_new-lam_old)/2};
              set vertex[1].z z-(lam_new-lam_old)/2; set vertex[10].z z+(lam_new-lam_old)/2;
              /*if phi < 90 then {foreach vertex vv where not fixed do {set vv.z z+z/abs(z)*(lam_new-lam_old)/2}}*/
              }
ar_op := {set vertex where fixed and z>lambda/2 constraint 5; set vertex where fixed and z<-lambda/2 constraint 1;
          set vertex where fixed and z=lambda/2 constraint 4; set vertex where fixed and z=-lambda/2 constraint 2;
          unset facet where on_constraint 1 fixed; 
          unset facet where on_constraint 3 fixed; 
          unset facet where on_constraint 5 fixed; 
          unset edge where on_constraint 1 fixed; 
          unset edge where on_constraint 3 fixed; 
          unset edge where on_constraint 5 fixed; 
          unset vertex where on_constraint 1 fixed;
          unset vertex where on_constraint 3 fixed;
          unset vertex where on_constraint 5 fixed;
          quiet on; {V 3; u}40; quiet off;
          set facet where on_constraint 1 fixed;
          set facet where on_constraint 3 fixed;
          set facet where on_constraint 5 fixed;
          set edge where on_constraint 1 fixed; 
          set edge where on_constraint 3 fixed; 
          set edge where on_constraint 5 fixed; 
          set edge where on_constraint 1 no_refine; 
          set edge where on_constraint 3 no_refine; 
          set edge where on_constraint 5 no_refine; 
          set vertex where on_constraint 1 fixed; 
          set vertex where on_constraint 3 fixed; 
          set vertex where on_constraint 5 fixed; 
          }
//          
// Refine tank wall mesh          
//
tank_wall_res_idx :=1          
vertex_number_fixed := 0
lr_fixed_max :=0
//
tank_wall_refine := {unset facet where on_constraint 1 fixed; unset facet where on_constraint 3 fixed; unset facet where on_constraint 5 fixed;
                     unset edge where on_constraint 1 fixed; unset edge where on_constraint 3 fixed; unset edge where on_constraint 5 fixed;
                     unset edge where on_constraint 1 no_refine; unset edge where on_constraint 3 no_refine; unset edge where on_constraint 5 no_refine;
                     unset vertex where on_constraint 1 fixed;  unset vertex where on_constraint 2 fixed; unset vertex where on_constraint 3 fixed;
                      unset vertex where on_constraint 4 fixed;  unset vertex where on_constraint 5 fixed; unset vertex where on_constraint 7 fixed;
                    set facet where on_constraint 14 fixed;set facet where on_constraint 15 fixed;set facet where on_constraint 16 fixed;
                      set edge where on_constraint 14 no_refine;set edge where on_constraint 15 no_refine;set edge where on_constraint 16 no_refine;
                     set edge where on_constraint 14 fixed;set edge where on_constraint 15 fixed;set edge where on_constraint 16 fixed;
                    set vertex where on_constraint 14 fixed;set vertex where on_constraint 15 fixed;set vertex where on_constraint 16 fixed;
                     lr_fixed := max(edge where not fixed, length); vertex_number_fixed_count := count(vertex where not fixed, id);
                     if tank_wall_res_idx == 1 then 
                     { while (vertex_number_fixed_count < vertex_number_fixed) do {lr_fixed := lr_fixed -0.01; print lr_fixed;
                                                                                   refine edge where not fixed and length > lr_fixed; quiet on;{V 3; u}40;quiet off;
                                                                                   vertex_number_fixed_count := count(vertex where not fixed, id); print vertex_number_fixed_count}
                     }
                     else { while (lr_fixed > lr_fixed_max) do { lr_fixed := lr_fixed - 0.01; print lr_fixed;
                                                                 refine edge where not fixed and length > lr_fixed; quiet on;{V 3; u}40;quiet off;
                                                                }};
                     set facet where on_constraint 1 fixed; set facet where on_constraint 3 fixed; set facet where on_constraint 5 fixed;
                     set edge where on_constraint 1 fixed; set edge where on_constraint 3 fixed; set edge where on_constraint 5 fixed;
                     set edge where on_constraint 1 no_refine; set edge where on_constraint 3 no_refine; set edge where on_constraint 5 no_refine;
                     set vertex where on_constraint 1 fixed;  set vertex where on_constraint 2 fixed; set vertex where on_constraint 3 fixed;
                     set vertex where on_constraint 4 fixed;  set vertex where on_constraint 5 fixed; set vertex where on_constraint 7 fixed;
                     unset facet where on_constraint 14 fixed; unset facet where on_constraint 15 fixed; unset facet where on_constraint 16 fixed;
                     unset edge where on_constraint 14 no_refine; unset edge where on_constraint 15 no_refine; unset edge where on_constraint 16 no_refine;
                     unset edge where on_constraint 14 fixed; unset edge where on_constraint 15 fixed; unset edge where on_constraint 16 fixed;    
                     unset vertex where on_constraint 14 fixed; unset vertex where on_constraint 15 fixed; unset vertex where on_constraint 16 fixed;
                     i
                     }
quiet off
